package hamster.kalah;

/**
 * Dieses BlueJ-Projekt ist ein sehr komplexes Beispiel, das aus der Welt der
 * Spiele-Programmierung stammt. Den Hamstern wird das Spielen beigebracht, so
 * dass sie in der Lage sind, gegen uns Menschen oder gegen andere Hamster so
 * genannte Zwei-Spieler-Strategiespiele wie Schach, Mhle, Dame, 4-Gewinnt oder
 * Reversi zu spielen, und die Hamster werden so gut sein, dass wir als Menschen
 * kaum noch eine Chance haben, gegen sie zu gewinnen.
 * 
 * Das Spiel, an dem das zugrunde liegende Prinzip demonstriert wird, nennt sich
 * Kalah. Kalah ist ein afrikanisches Brettspiel fr zwei Spieler. Es besteht
 * aus einem Brett mit zwei Reihen von je sechs Mulden, in denen anfangs je
 * sechs Steine liegen, und zwei weiteren Mulden, die Kalah heien und anfangs
 * leer sind. Spieler A gehren die oberen Lcher und die linke Kalah, Spieler B
 * die unteren Lcher und die rechte Kalah.
 * 
 * Ziel der beiden Spieler ist es, im Laufe eines Spiels mglichst viele Steine
 * in die eigene Kalah zu bringen. Spieler A beginnt das Spiel. Ein Spielzug
 * luft dabei wie folgt ab: (1) Der Spieler whlt eine seiner Mulden (nicht
 * seine Kalah) aus, die nicht leer ist, entleert sie und verteilt auf die
 * entgegen dem Uhrzeigersinn folgenden Mulden je einen Stein, wobei er die
 * Kalah des Gegners auslsst. (2) Landet der letzte Stein in einer eigenen
 * leeren Mulde (keine Kalah) und ist die gegenberliegende Mulde des Gegners
 * nicht leer, so kommen der Stein sowie alle Steine der gegenberliegenden
 * Mulde in die eigene Kalah. (3) Landet der letzte Stein in der eigenen Kalah,
 * so muss der Spieler noch einmal ziehen. Ansonsten ist der Gegner an der
 * Reihe.
 * 
 * Das Spiel ist beendet, wenn ein Spieler an der Reihe ist, aber keinen
 * erlaubten Zug mehr ausfhren kann, d.h. alle seine Mulden leer sind. Der
 * andere Spieler darf dann alle Steine, die noch in seinen Mulden liegen, in
 * die eigene Kalah legen. Gewonnen hat der Spieler, der anschlieend mehr
 * Steine in seiner Kalah liegen hat. Bei Gleichstand endet das Spiel mit einem
 * Unentschieden.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (29.01.2006)
 * 
 */
public class KalahSpiel {

	/**
	 * Ein moegliches Spiel
	 */
	public static void main(String[] args) {
		KalahSpiel spiel = new KalahSpiel();
		Spieler sA = new StrategieHamster(3);
		Spieler sB = new StrategieHamster(1);
		spiel.spielen(sA, sB);
	}

	/**
	 * initialisieren eines Kalah-Spiels
	 */
	public KalahSpiel() {
	}

	private SchiedsrichterHamster schiedsrichter;

	/**
	 * Spielen eines Kalah-Spiels
	 * 
	 * @param spielerA
	 *            der Spieler, der Spieler A sein soll
	 * @param spielerB
	 *            der Spieler, der Spieler B sein soll
	 */
	public void spielen(Spieler spielerA, Spieler spielerB) {

		// Spielvorbereitung
		schiedsrichter = new SchiedsrichterHamster();
		spielerA.setzeSpieler(true);
		spielerB.setzeSpieler(false);
		schiedsrichter.initialisiereSpiel(spielerA, spielerB);
		spielerA.begibDichAufAusgangsKachel();
		spielerB.begibDichAufAusgangsKachel();

		// es wird gemaess der Spielregeln gezogen;
		// nach jedem Spielzug kontrolliert der Schiedsrichter
		Spieler aktuellerSpieler = schiedsrichter.werIstAmZug();
		while (true) {
			Spielzug zug;
			try {
				zug = aktuellerSpieler.naechsterSpielzug();
			} catch (Exception exc) {

				// wenn waehrend der Ausfuehrung der Methode ein
				// Fehler auftritt, hat der Spieler verloren
				schiedsrichter.schreib("Spieler "
						+ aktuellerSpieler.liefereSpielerTyp()
						+ " hat die folgende Exception erzeugt\n"
						+ " und damit verloren: " + exc.toString());
				return; // Spielende
			}

			if (!schiedsrichter.istSpielzugOk(aktuellerSpieler, zug)) {
				schiedsrichter.schreib("Spieler "
						+ aktuellerSpieler.liefereSpielerTyp()
						+ " hat sich nicht an die\n"
						+ "Spielregeln gehalten und damit\n" + "verloren!");
				return; // Spielende
			}

			if (schiedsrichter.istSpielBeendet()) {

				// die verbliebenen Koerner muessen noch in die Kalahs
				// gepackt werden
				spielerA.sammleResteUndBringSieZumKalah();
				if (!schiedsrichter.istEndzustandVonSpielerAOk()) {
					schiedsrichter.schreib("Spieler A"
							+ " hat sich nicht an die\n"
							+ "Spielregeln gehalten und damit\n" + "verloren!");
					return; // Spielende
				}

				spielerB.sammleResteUndBringSieZumKalah();
				if (!schiedsrichter.istEndzustandVonSpielerBOk()) {
					schiedsrichter.schreib("Spieler B"
							+ " hat sich nicht an die\n"
							+ "Spielregeln gehalten und damit\n" + "verloren!");
					return; // Spielende
				}

				// Siegerehrung
				Spieler sieger = schiedsrichter.liefereSieger();
				if (sieger == null) {
					schiedsrichter.schreib("Unentschieden!");

					// zur Belohnung duerfen die Hamster noch die Koerner in
					// ihren Kalahs fressen
					aktuellerSpieler.spielBeenden(false);
					Spieler gegner = schiedsrichter
							.liefereGegner(aktuellerSpieler);
					gegner.spielBeenden(false);
				} else {
					int punkteA = schiedsrichter.getPunkteA();
					int punkteB = schiedsrichter.getPunkteB();
					schiedsrichter.schreib("Endstand: " + punkteA + " : "
							+ punkteB + "\n" + "Spieler "
							+ sieger.liefereSpielerTyp() + " hat gewonnen!");

					// zur Belohnung duerfen die Hamster noch die Koerner in
					// ihren Kalahs fressen
					sieger.spielBeenden(true);
					Spieler gegner = schiedsrichter.liefereGegner(sieger);
					gegner.spielBeenden(false);
				}

				return; // Spielende
			}

			// naechste Runde
			aktuellerSpieler = schiedsrichter.werIstAmZug();
		}
	}

}
